//***************************************************************************
// Class to manage application settings
//***************************************************************************
function AppConfig(storeType)
{
  
  this.store = 'ELS';
  if(storeType != null)
    this.store = storeType; //by default use Adobe's Encrypted Local Store

  this.fileCache = ''; // Currently used when configuration is stored in a file
  this.fileLoaded = false;


  this.getStringParameter = function(name) {
    if(this.store == 'COOKIE') return this.getCookie(name);
    else if(this.store == 'FILE') return this.getFileParameter(name);
    var storedValue = null;
    try
    {
      storedValue = air.EncryptedLocalStore.getItem(name);
    }
    catch(e){}
    var value;
    if(storedValue != null && storedValue.length > 0) 
      value = storedValue.readUTFBytes(storedValue.length);
    else
      value = ''; //default - ''
    return value;
  }

  this.getBoolParameter = function(name) {
    var storedValue;

    if(this.store == 'COOKIE') storedValue = this.getCookie(name);
    else if(this.store == 'FILE') storedValue = this.getFileParameter(name);
    else storedValue = this.getStringParameter(name);

    if(storedValue == 'true') 
      return true;
    else
      return false;
  }

  this.getIntParameter = function(name) {
    if(this.store == 'COOKIE') return this.getIntCookie(name);
    else if(this.store == 'FILE') return this.getIntFileParameter(name);
    var storedValue = null;
    try
    {
      storedValue = air.EncryptedLocalStore.getItem(name);
    }
    catch(e){}
    var value;
    if(storedValue != null && storedValue.length > 0) 
      value = storedValue.readUTFBytes(storedValue.length);
    else
      value = 0; //default - 0
    return value;
  }

  this.getIntCookie = function(name) {
    var storedValue = this.getCookie(name);
    var value;
    if(storedValue != null && storedValue.length > 0) 
      value = storedValue;
    else
      value = 0; //default - 0
    return value;
  }

  this.getIntFileParameter = function(name)
  {
    var storedValue = this.getFileParameter(name);
    var value;
    if(storedValue != null && storedValue.length > 0) 
      value = storedValue;
    else
      value = 0; //default - 0
    return value;
  }

  this.setParameter = function(name, value) {
    if(this.store == 'COOKIE') 
    {
      this.setCookie(name, value);
      return;
    }
    else if(this.store == 'FILE')
    {
      this.setFileParameter(name, value);
      return;
    }
    try
    {
      var settBytes = new air.ByteArray();
      settBytes.writeUTFBytes(value); 
      air.EncryptedLocalStore.setItem(name, settBytes);
    }
    catch(e)
    {
      alert('Error writing configuration in ELS: [' + e + ']');
    }
  }
 
  this.commit = function() {
    if(this.store == 'FILE')
    {
      try
      {
        //alert('COMMIT: [' + this.fileCache + ']');
        var file = air.File.applicationStorageDirectory.resolvePath("wizehive.cfg"); 
        var fileStr = new air.FileStream(); 
        fileStr.open(file, air.FileMode.UPDATE);
        fileStr.writeMultiByte(this.fileCache, "iso-8859-1");
        fileStr.close();
        this.fileCache = '';
        this.fileLoaded = false;
      }
      catch(e)
      {
        alert('Error writing configuration: [' + e + ']');
      };
    }
  }

  // Using cookies as a local store
  this.getCookie=function(Name){ 
	var re=new RegExp(Name+"=[^;]+", "i"); //construct RE to search for target name/value pair
	if (document.cookie.match(re)) //if cookie found
		return document.cookie.match(re)[0].split("=")[1]; //return its value
	return "";
  }

  this.setCookie=function(name, value){
	document.cookie = name+"="+value+";path=/"; //cookie value is domain wide (path=/)
  }

  this.getFileParameter=function(name){
    if(!this.fileLoaded)
    {
      this.loadFileConfiguration();
    }
    var value = '';
    var separator = ";";
    if(this.fileCache.indexOf(name) != -1)
    {
      // Extract name=value
      value = this.fileCache.substring(this.fileCache.indexOf(name), this.fileCache.indexOf(';', this.fileCache.indexOf(name)));
      // Extract value
      if(value.indexOf('=') != -1)
        value = value.substring(value.indexOf('=') + 1).trim();
      else
        value = '';
    }
    //alert(name + '=[' + value + ']');
    return value; 
  }

  this.setFileParameter=function(name, value)
  {
    if(!this.fileLoaded) this.loadFileConfiguration();
    // Check if the parameter exists
    if(this.fileCache.indexOf(name) != -1)
    {
      var newPair = name + "=" + value;
      var beforePair = this.fileCache.substring(0, this.fileCache.indexOf(name));
      if((this.fileCache.indexOf(name) > 0) && (beforePair.charAt(beforePair.length - 1) != '\n'))
        newPair = "\n" + newPair;
      // Replace the value
      this.fileCache =  beforePair + newPair +
                  this.fileCache.substring(this.fileCache.indexOf(';', this.fileCache.indexOf(name)));
    }
    else
    {
      // Append to the end
      if(this.fileCache.length > 0)
        this.fileCache += "\n" + name + "=" + value + ";";
      else
        this.fileCache = name + "=" + value + ";";
    }
  }

  this.loadFileConfiguration=function() {
    try
    {
      var file = air.File.applicationStorageDirectory.resolvePath("wizehive.cfg"); 
      var fileStr = new air.FileStream(); 
      fileStr.open(file, air.FileMode.UPDATE);
      this.fileCache = fileStr.readMultiByte(fileStr.bytesAvailable, "iso-8859-1");
      // Remove last new line character
      if((this.fileCache.length > 1) && (this.fileCache.charAt(this.fileCache.length - 1) == '\n'))
        this.fileCache = this.fileCache.substring(0, this.fileCache.length - 1); 
      fileStr.close();
      this.fileLoaded = true;
    }
    catch(e)
    {
      alert('Error loading file configuration: [' + e + ']');
      this.fileCache = '';
      this.fileLoaded = false;
    };
  }
}

String.prototype.trim = function () {
    return this.replace(/^\s*/, "").replace(/\s*$/, "");
}